import { Injectable, Inject } from '@angular/core';

import { DeviceType, DesktopType } from '@ucap/core';
import { NativeService, NativeType, OsType } from '@ucap/native';

import { I18nService } from '@ucap/ng-i18n';
import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';
import { SessionStorageService } from '@ucap/ng-web-storage';
import { LoginSession } from '@app/models/login-session';
import { AppKey } from '@app/types/app-key.type';

@Injectable()
export class AppService {
  constructor(
    private sessionStorageService: SessionStorageService,
    private i18nService: I18nService,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService
  ) {}

  initialize(): Promise<void[]> {
    const initSession = new Promise<void>(async (resolve, reject) => {
      try {
        let deviceType: DeviceType;
        let desktopType: DesktopType;

        switch (this.nativeService.type()) {
          case NativeType.Browser:
            deviceType = DeviceType.Web;
            break;
          case NativeType.Electron:
            deviceType = DeviceType.PC;
            break;
          default:
            break;
        }

        const osType = await this.nativeService.osType();

        switch (osType) {
          case OsType.Windows:
            desktopType = DesktopType.Windows;
            break;
          case OsType.MacOS:
            desktopType = DesktopType.MacOS;
            break;
          default:
            desktopType = DesktopType.Linux;
            break;
        }

        this.sessionStorageService.set<LoginSession>(AppKey.LoginSession, {
          deviceType,
          desktopType
        });

        resolve();
      } catch (error) {
        reject(error);
      }
    });

    const initI18n = new Promise<void>(async (resolve, reject) => {
      switch (this.nativeService.type()) {
        case NativeType.Browser:
          const xhr = await import('i18next-xhr-backend').then(m => m.default);
          const languageDetector = await import(
            'i18next-browser-languagedetector'
          ).then(m => m.default);
          this.i18nService.use(xhr).use(languageDetector);
          break;
        // case NativeType.Electron:
        //   const nodeFs = await import('i18next-node-fs-backend').then(m => m);
        //   i18nService.use(nodeFs);
        //   break;
        default:
          break;
      }

      this.i18nService
        .init({
          whitelist: ['ko', 'en'],
          fallbackLng: 'ko',
          debug: true, // set debug?
          returnEmptyString: false,
          ns: [
            'common',
            'organization',
            'authentication',
            'group',
            'chat',
            'call',
            'message'
          ],
          backend: {
            loadPath: 'assets/i18n/{{lng}}/{{ns}}.json'
          }
        })
        .then(() => {
          resolve();
        })
        .catch(reason => {
          reject(reason);
        });
    });

    return Promise.all([initSession, initI18n]);
  }
}
